# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, cast
from typing_extensions import Literal

import httpx

from .ips import (
    IPsResource,
    AsyncIPsResource,
    IPsResourceWithRawResponse,
    AsyncIPsResourceWithRawResponse,
    IPsResourceWithStreamingResponse,
    AsyncIPsResourceWithStreamingResponse,
)
from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.networks import route_edit_params, route_list_params, route_create_params
from .....types.zero_trust.networks.route import Route
from .....types.zero_trust.networks.teamnet import Teamnet

__all__ = ["RoutesResource", "AsyncRoutesResource"]


class RoutesResource(SyncAPIResource):
    @cached_property
    def ips(self) -> IPsResource:
        return IPsResource(self._client)

    @cached_property
    def networks(self) -> NetworksResource:
        return NetworksResource(self._client)

    @cached_property
    def with_raw_response(self) -> RoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RoutesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        network: str,
        tunnel_id: str,
        comment: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Routes a private network through a Cloudflare Tunnel.

        Args:
          account_id: Cloudflare account ID

          network: The private IPv4 or IPv6 range connected by the route, in CIDR notation.

          tunnel_id: UUID of the tunnel.

          comment: Optional remark describing the route.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/teamnet/routes", account_id=account_id),
            body=maybe_transform(
                {
                    "network": network,
                    "tunnel_id": tunnel_id,
                    "comment": comment,
                    "virtual_network_id": virtual_network_id,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    def list(
        self,
        *,
        account_id: str,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        network_subset: str | Omit = omit,
        network_superset: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        route_id: str | Omit = omit,
        tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]] | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Teamnet]:
        """
        Lists and filters private network routes in an account.

        Args:
          account_id: Cloudflare account ID

          comment: Optional remark describing the route.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted routes. If `false`, exclude deleted routes. If
              empty, all routes will be included.

          network_subset: If set, only list routes that are contained within this IP range.

          network_superset: If set, only list routes that contain this IP range.

          page: Page number of paginated results.

          per_page: Number of results to display.

          route_id: UUID of the route.

          tun_types: The types of tunnels to filter by, separated by commas.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/teamnet/routes", account_id=account_id),
            page=SyncV4PagePaginationArray[Teamnet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "comment": comment,
                        "existed_at": existed_at,
                        "is_deleted": is_deleted,
                        "network_subset": network_subset,
                        "network_superset": network_superset,
                        "page": page,
                        "per_page": per_page,
                        "route_id": route_id,
                        "tun_types": tun_types,
                        "tunnel_id": tunnel_id,
                        "virtual_network_id": virtual_network_id,
                    },
                    route_list_params.RouteListParams,
                ),
            ),
            model=Teamnet,
        )

    def delete(
        self,
        route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Deletes a private network route from an account.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    def edit(
        self,
        route_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        network: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Updates an existing private network route in an account.

        The fields that are
        meant to be updated should be provided in the body of the request.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          comment: Optional remark describing the route.

          network: The private IPv4 or IPv6 range connected by the route, in CIDR notation.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            body=maybe_transform(
                {
                    "comment": comment,
                    "network": network,
                    "tunnel_id": tunnel_id,
                    "virtual_network_id": virtual_network_id,
                },
                route_edit_params.RouteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    def get(
        self,
        route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Get a private network route in an account.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )


class AsyncRoutesResource(AsyncAPIResource):
    @cached_property
    def ips(self) -> AsyncIPsResource:
        return AsyncIPsResource(self._client)

    @cached_property
    def networks(self) -> AsyncNetworksResource:
        return AsyncNetworksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRoutesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoutesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoutesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRoutesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        network: str,
        tunnel_id: str,
        comment: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Routes a private network through a Cloudflare Tunnel.

        Args:
          account_id: Cloudflare account ID

          network: The private IPv4 or IPv6 range connected by the route, in CIDR notation.

          tunnel_id: UUID of the tunnel.

          comment: Optional remark describing the route.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/teamnet/routes", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "network": network,
                    "tunnel_id": tunnel_id,
                    "comment": comment,
                    "virtual_network_id": virtual_network_id,
                },
                route_create_params.RouteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    def list(
        self,
        *,
        account_id: str,
        comment: str | Omit = omit,
        existed_at: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        network_subset: str | Omit = omit,
        network_superset: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        route_id: str | Omit = omit,
        tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]] | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Teamnet, AsyncV4PagePaginationArray[Teamnet]]:
        """
        Lists and filters private network routes in an account.

        Args:
          account_id: Cloudflare account ID

          comment: Optional remark describing the route.

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted routes. If `false`, exclude deleted routes. If
              empty, all routes will be included.

          network_subset: If set, only list routes that are contained within this IP range.

          network_superset: If set, only list routes that contain this IP range.

          page: Page number of paginated results.

          per_page: Number of results to display.

          route_id: UUID of the route.

          tun_types: The types of tunnels to filter by, separated by commas.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/teamnet/routes", account_id=account_id),
            page=AsyncV4PagePaginationArray[Teamnet],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "comment": comment,
                        "existed_at": existed_at,
                        "is_deleted": is_deleted,
                        "network_subset": network_subset,
                        "network_superset": network_superset,
                        "page": page,
                        "per_page": per_page,
                        "route_id": route_id,
                        "tun_types": tun_types,
                        "tunnel_id": tunnel_id,
                        "virtual_network_id": virtual_network_id,
                    },
                    route_list_params.RouteListParams,
                ),
            ),
            model=Teamnet,
        )

    async def delete(
        self,
        route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Deletes a private network route from an account.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    async def edit(
        self,
        route_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        network: str | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Updates an existing private network route in an account.

        The fields that are
        meant to be updated should be provided in the body of the request.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          comment: Optional remark describing the route.

          network: The private IPv4 or IPv6 range connected by the route, in CIDR notation.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "network": network,
                    "tunnel_id": tunnel_id,
                    "virtual_network_id": virtual_network_id,
                },
                route_edit_params.RouteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    async def get(
        self,
        route_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """
        Get a private network route in an account.

        Args:
          account_id: Cloudflare account ID

          route_id: UUID of the route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not route_id:
            raise ValueError(f"Expected a non-empty value for `route_id` but received {route_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/teamnet/routes/{route_id}", account_id=account_id, route_id=route_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )


class RoutesResourceWithRawResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_raw_response_wrapper(
            routes.create,
        )
        self.list = to_raw_response_wrapper(
            routes.list,
        )
        self.delete = to_raw_response_wrapper(
            routes.delete,
        )
        self.edit = to_raw_response_wrapper(
            routes.edit,
        )
        self.get = to_raw_response_wrapper(
            routes.get,
        )

    @cached_property
    def ips(self) -> IPsResourceWithRawResponse:
        return IPsResourceWithRawResponse(self._routes.ips)

    @cached_property
    def networks(self) -> NetworksResourceWithRawResponse:
        return NetworksResourceWithRawResponse(self._routes.networks)


class AsyncRoutesResourceWithRawResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_raw_response_wrapper(
            routes.create,
        )
        self.list = async_to_raw_response_wrapper(
            routes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            routes.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            routes.edit,
        )
        self.get = async_to_raw_response_wrapper(
            routes.get,
        )

    @cached_property
    def ips(self) -> AsyncIPsResourceWithRawResponse:
        return AsyncIPsResourceWithRawResponse(self._routes.ips)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithRawResponse:
        return AsyncNetworksResourceWithRawResponse(self._routes.networks)


class RoutesResourceWithStreamingResponse:
    def __init__(self, routes: RoutesResource) -> None:
        self._routes = routes

        self.create = to_streamed_response_wrapper(
            routes.create,
        )
        self.list = to_streamed_response_wrapper(
            routes.list,
        )
        self.delete = to_streamed_response_wrapper(
            routes.delete,
        )
        self.edit = to_streamed_response_wrapper(
            routes.edit,
        )
        self.get = to_streamed_response_wrapper(
            routes.get,
        )

    @cached_property
    def ips(self) -> IPsResourceWithStreamingResponse:
        return IPsResourceWithStreamingResponse(self._routes.ips)

    @cached_property
    def networks(self) -> NetworksResourceWithStreamingResponse:
        return NetworksResourceWithStreamingResponse(self._routes.networks)


class AsyncRoutesResourceWithStreamingResponse:
    def __init__(self, routes: AsyncRoutesResource) -> None:
        self._routes = routes

        self.create = async_to_streamed_response_wrapper(
            routes.create,
        )
        self.list = async_to_streamed_response_wrapper(
            routes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            routes.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            routes.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            routes.get,
        )

    @cached_property
    def ips(self) -> AsyncIPsResourceWithStreamingResponse:
        return AsyncIPsResourceWithStreamingResponse(self._routes.ips)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithStreamingResponse:
        return AsyncNetworksResourceWithStreamingResponse(self._routes.networks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rule_list_response import RuleListResponse as RuleListResponse
from .rule_update_params import RuleUpdateParams as RuleUpdateParams
from .rule_update_response import RuleUpdateResponse as RuleUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .dhcp_relay import DHCPRelay
from .dhcp_server import DHCPServer

__all__ = ["LANStaticAddressing"]


class LANStaticAddressing(BaseModel):
    """
    If the site is not configured in high availability mode, this configuration is optional (if omitted, use DHCP). However, if in high availability mode, static_address is required along with secondary and virtual address.
    """

    address: str
    """A valid CIDR notation representing an IP range."""

    dhcp_relay: Optional[DHCPRelay] = None

    dhcp_server: Optional[DHCPServer] = None

    secondary_address: Optional[str] = None
    """A valid CIDR notation representing an IP range."""

    virtual_address: Optional[str] = None
    """A valid CIDR notation representing an IP range."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Union, Iterable, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ai_gateway import log_edit_params, log_list_params, log_delete_params
from ...types.ai_gateway.log_get_response import LogGetResponse
from ...types.ai_gateway.log_list_response import LogListResponse
from ...types.ai_gateway.log_delete_response import LogDeleteResponse

__all__ = ["LogsResource", "AsyncLogsResource"]


class LogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogsResourceWithStreamingResponse(self)

    def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        cached: bool | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        feedback: Literal[0, 1] | Omit = omit,
        filters: Iterable[log_list_params.Filter] | Omit = omit,
        max_cost: float | Omit = omit,
        max_duration: float | Omit = omit,
        max_tokens_in: float | Omit = omit,
        max_tokens_out: float | Omit = omit,
        max_total_tokens: float | Omit = omit,
        meta_info: bool | Omit = omit,
        min_cost: float | Omit = omit,
        min_duration: float | Omit = omit,
        min_tokens_in: float | Omit = omit,
        min_tokens_out: float | Omit = omit,
        min_total_tokens: float | Omit = omit,
        model: str | Omit = omit,
        model_type: str | Omit = omit,
        order_by: Literal["created_at", "provider", "model", "model_type", "success", "cached"] | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        provider: str | Omit = omit,
        request_content_type: str | Omit = omit,
        response_content_type: str | Omit = omit,
        search: str | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        success: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[LogListResponse]:
        """
        List Gateway Logs

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            page=SyncV4PagePaginationArray[LogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cached": cached,
                        "direction": direction,
                        "end_date": end_date,
                        "feedback": feedback,
                        "filters": filters,
                        "max_cost": max_cost,
                        "max_duration": max_duration,
                        "max_tokens_in": max_tokens_in,
                        "max_tokens_out": max_tokens_out,
                        "max_total_tokens": max_total_tokens,
                        "meta_info": meta_info,
                        "min_cost": min_cost,
                        "min_duration": min_duration,
                        "min_tokens_in": min_tokens_in,
                        "min_tokens_out": min_tokens_out,
                        "min_total_tokens": min_total_tokens,
                        "model": model,
                        "model_type": model_type,
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                        "page": page,
                        "per_page": per_page,
                        "provider": provider,
                        "request_content_type": request_content_type,
                        "response_content_type": response_content_type,
                        "search": search,
                        "start_date": start_date,
                        "success": success,
                    },
                    log_list_params.LogListParams,
                ),
            ),
            model=LogListResponse,
        )

    def delete(
        self,
        gateway_id: str,
        *,
        account_id: str,
        filters: Iterable[log_delete_params.Filter] | Omit = omit,
        limit: int | Omit = omit,
        order_by: Literal[
            "created_at",
            "provider",
            "model",
            "model_type",
            "success",
            "cached",
            "cost",
            "tokens_in",
            "tokens_out",
            "duration",
            "feedback",
        ]
        | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogDeleteResponse:
        """
        Delete Gateway Logs

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filters": filters,
                        "limit": limit,
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                    },
                    log_delete_params.LogDeleteParams,
                ),
            ),
            cast_to=LogDeleteResponse,
        )

    def edit(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        feedback: Optional[float] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        score: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Updates metadata for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=maybe_transform(
                {
                    "feedback": feedback,
                    "metadata": metadata,
                    "score": score,
                },
                log_edit_params.LogEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogGetResponse:
        """
        Retrieves detailed information for a specific AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LogGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[LogGetResponse], ResultWrapper[LogGetResponse]),
        )

    def request(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves the original request payload for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}/request",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def response(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves the response payload for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}/response",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogsResourceWithStreamingResponse(self)

    def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        cached: bool | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        feedback: Literal[0, 1] | Omit = omit,
        filters: Iterable[log_list_params.Filter] | Omit = omit,
        max_cost: float | Omit = omit,
        max_duration: float | Omit = omit,
        max_tokens_in: float | Omit = omit,
        max_tokens_out: float | Omit = omit,
        max_total_tokens: float | Omit = omit,
        meta_info: bool | Omit = omit,
        min_cost: float | Omit = omit,
        min_duration: float | Omit = omit,
        min_tokens_in: float | Omit = omit,
        min_tokens_out: float | Omit = omit,
        min_total_tokens: float | Omit = omit,
        model: str | Omit = omit,
        model_type: str | Omit = omit,
        order_by: Literal["created_at", "provider", "model", "model_type", "success", "cached"] | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        provider: str | Omit = omit,
        request_content_type: str | Omit = omit,
        response_content_type: str | Omit = omit,
        search: str | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        success: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LogListResponse, AsyncV4PagePaginationArray[LogListResponse]]:
        """
        List Gateway Logs

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            page=AsyncV4PagePaginationArray[LogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cached": cached,
                        "direction": direction,
                        "end_date": end_date,
                        "feedback": feedback,
                        "filters": filters,
                        "max_cost": max_cost,
                        "max_duration": max_duration,
                        "max_tokens_in": max_tokens_in,
                        "max_tokens_out": max_tokens_out,
                        "max_total_tokens": max_total_tokens,
                        "meta_info": meta_info,
                        "min_cost": min_cost,
                        "min_duration": min_duration,
                        "min_tokens_in": min_tokens_in,
                        "min_tokens_out": min_tokens_out,
                        "min_total_tokens": min_total_tokens,
                        "model": model,
                        "model_type": model_type,
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                        "page": page,
                        "per_page": per_page,
                        "provider": provider,
                        "request_content_type": request_content_type,
                        "response_content_type": response_content_type,
                        "search": search,
                        "start_date": start_date,
                        "success": success,
                    },
                    log_list_params.LogListParams,
                ),
            ),
            model=LogListResponse,
        )

    async def delete(
        self,
        gateway_id: str,
        *,
        account_id: str,
        filters: Iterable[log_delete_params.Filter] | Omit = omit,
        limit: int | Omit = omit,
        order_by: Literal[
            "created_at",
            "provider",
            "model",
            "model_type",
            "success",
            "cached",
            "cost",
            "tokens_in",
            "tokens_out",
            "duration",
            "feedback",
        ]
        | Omit = omit,
        order_by_direction: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogDeleteResponse:
        """
        Delete Gateway Logs

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filters": filters,
                        "limit": limit,
                        "order_by": order_by,
                        "order_by_direction": order_by_direction,
                    },
                    log_delete_params.LogDeleteParams,
                ),
            ),
            cast_to=LogDeleteResponse,
        )

    async def edit(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        feedback: Optional[float] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        score: Optional[float] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Updates metadata for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            body=await async_maybe_transform(
                {
                    "feedback": feedback,
                    "metadata": metadata,
                    "score": score,
                },
                log_edit_params.LogEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogGetResponse:
        """
        Retrieves detailed information for a specific AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LogGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[LogGetResponse], ResultWrapper[LogGetResponse]),
        )

    async def request(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves the original request payload for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}/request",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def response(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves the response payload for an AI Gateway log entry.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/logs/{id}/response",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class LogsResourceWithRawResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

        self.list = to_raw_response_wrapper(
            logs.list,
        )
        self.delete = to_raw_response_wrapper(
            logs.delete,
        )
        self.edit = to_raw_response_wrapper(
            logs.edit,
        )
        self.get = to_raw_response_wrapper(
            logs.get,
        )
        self.request = to_raw_response_wrapper(
            logs.request,
        )
        self.response = to_raw_response_wrapper(
            logs.response,
        )


class AsyncLogsResourceWithRawResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

        self.list = async_to_raw_response_wrapper(
            logs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            logs.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            logs.edit,
        )
        self.get = async_to_raw_response_wrapper(
            logs.get,
        )
        self.request = async_to_raw_response_wrapper(
            logs.request,
        )
        self.response = async_to_raw_response_wrapper(
            logs.response,
        )


class LogsResourceWithStreamingResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

        self.list = to_streamed_response_wrapper(
            logs.list,
        )
        self.delete = to_streamed_response_wrapper(
            logs.delete,
        )
        self.edit = to_streamed_response_wrapper(
            logs.edit,
        )
        self.get = to_streamed_response_wrapper(
            logs.get,
        )
        self.request = to_streamed_response_wrapper(
            logs.request,
        )
        self.response = to_streamed_response_wrapper(
            logs.response,
        )


class AsyncLogsResourceWithStreamingResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

        self.list = async_to_streamed_response_wrapper(
            logs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            logs.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            logs.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            logs.get,
        )
        self.request = async_to_streamed_response_wrapper(
            logs.request,
        )
        self.response = async_to_streamed_response_wrapper(
            logs.response,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["DomainListParams"]


class DomainListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    active_delivery_mode: Literal["DIRECT", "BCC", "JOURNAL", "API", "RETRO_SCAN"]
    """Currently active delivery mode to filter by."""

    allowed_delivery_mode: Literal["DIRECT", "BCC", "JOURNAL", "API", "RETRO_SCAN"]
    """Delivery mode to filter by."""

    direction: Literal["asc", "desc"]
    """The sorting direction."""

    domain: SequenceNotStr[str]
    """Domain names to filter by."""

    integration_id: str
    """Integration ID to filter by."""

    order: Literal["domain", "created_at"]
    """Field to sort by."""

    page: int
    """Current page within paginated list of results."""

    per_page: int
    """The number of results per page. Maximum value is 1000."""

    search: str
    """Search term for filtering records. Behavior may change."""

    status: Literal["pending", "active", "failed", "timeout"]
    """Filters response to domains with the provided status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .custom.custom import (
    CustomResource,
    AsyncCustomResource,
    CustomResourceWithRawResponse,
    AsyncCustomResourceWithRawResponse,
    CustomResourceWithStreamingResponse,
    AsyncCustomResourceWithStreamingResponse,
)
from .default.default import (
    DefaultResource,
    AsyncDefaultResource,
    DefaultResourceWithRawResponse,
    AsyncDefaultResourceWithRawResponse,
    DefaultResourceWithStreamingResponse,
    AsyncDefaultResourceWithStreamingResponse,
)

__all__ = ["PoliciesResource", "AsyncPoliciesResource"]


class PoliciesResource(SyncAPIResource):
    @cached_property
    def default(self) -> DefaultResource:
        return DefaultResource(self._client)

    @cached_property
    def custom(self) -> CustomResource:
        return CustomResource(self._client)

    @cached_property
    def with_raw_response(self) -> PoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PoliciesResourceWithStreamingResponse(self)


class AsyncPoliciesResource(AsyncAPIResource):
    @cached_property
    def default(self) -> AsyncDefaultResource:
        return AsyncDefaultResource(self._client)

    @cached_property
    def custom(self) -> AsyncCustomResource:
        return AsyncCustomResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPoliciesResourceWithStreamingResponse(self)


class PoliciesResourceWithRawResponse:
    def __init__(self, policies: PoliciesResource) -> None:
        self._policies = policies

    @cached_property
    def default(self) -> DefaultResourceWithRawResponse:
        return DefaultResourceWithRawResponse(self._policies.default)

    @cached_property
    def custom(self) -> CustomResourceWithRawResponse:
        return CustomResourceWithRawResponse(self._policies.custom)


class AsyncPoliciesResourceWithRawResponse:
    def __init__(self, policies: AsyncPoliciesResource) -> None:
        self._policies = policies

    @cached_property
    def default(self) -> AsyncDefaultResourceWithRawResponse:
        return AsyncDefaultResourceWithRawResponse(self._policies.default)

    @cached_property
    def custom(self) -> AsyncCustomResourceWithRawResponse:
        return AsyncCustomResourceWithRawResponse(self._policies.custom)


class PoliciesResourceWithStreamingResponse:
    def __init__(self, policies: PoliciesResource) -> None:
        self._policies = policies

    @cached_property
    def default(self) -> DefaultResourceWithStreamingResponse:
        return DefaultResourceWithStreamingResponse(self._policies.default)

    @cached_property
    def custom(self) -> CustomResourceWithStreamingResponse:
        return CustomResourceWithStreamingResponse(self._policies.custom)


class AsyncPoliciesResourceWithStreamingResponse:
    def __init__(self, policies: AsyncPoliciesResource) -> None:
        self._policies = policies

    @cached_property
    def default(self) -> AsyncDefaultResourceWithStreamingResponse:
        return AsyncDefaultResourceWithStreamingResponse(self._policies.default)

    @cached_property
    def custom(self) -> AsyncCustomResourceWithStreamingResponse:
        return AsyncCustomResourceWithStreamingResponse(self._policies.custom)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CORSGetResponse", "Rule", "RuleAllowed"]


class RuleAllowed(BaseModel):
    """Object specifying allowed origins, methods and headers for this CORS rule."""

    methods: List[Literal["GET", "PUT", "POST", "DELETE", "HEAD"]]
    """
    Specifies the value for the Access-Control-Allow-Methods header R2 sets when
    requesting objects in a bucket from a browser.
    """

    origins: List[str]
    """
    Specifies the value for the Access-Control-Allow-Origin header R2 sets when
    requesting objects in a bucket from a browser.
    """

    headers: Optional[List[str]] = None
    """
    Specifies the value for the Access-Control-Allow-Headers header R2 sets when
    requesting objects in this bucket from a browser. Cross-origin requests that
    include custom headers (e.g. x-user-id) should specify these headers as
    AllowedHeaders.
    """


class Rule(BaseModel):
    allowed: RuleAllowed
    """Object specifying allowed origins, methods and headers for this CORS rule."""

    id: Optional[str] = None
    """Identifier for this rule."""

    expose_headers: Optional[List[str]] = FieldInfo(alias="exposeHeaders", default=None)
    """
    Specifies the headers that can be exposed back, and accessed by, the JavaScript
    making the cross-origin request. If you need to access headers beyond the
    safelisted response headers, such as Content-Encoding or cf-cache-status, you
    must specify it here.
    """

    max_age_seconds: Optional[float] = FieldInfo(alias="maxAgeSeconds", default=None)
    """
    Specifies the amount of time (in seconds) browsers are allowed to cache CORS
    preflight responses. Browsers may limit this to 2 hours or less, even if the
    maximum value (86400) is specified.
    """


class CORSGetResponse(BaseModel):
    rules: Optional[List[Rule]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr
from .hostname_association import HostnameAssociation

__all__ = ["HostnameAssociationUpdateParams"]


class HostnameAssociationUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    hostnames: SequenceNotStr[HostnameAssociation]

    mtls_certificate_id: str
    """
    The UUID for a certificate that was uploaded to the mTLS Certificate Management
    endpoint. If no mtls_certificate_id is given, the hostnames will be associated
    to your active Cloudflare Managed CA.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.user.tokens import permission_group_list_params
from ....types.user.tokens.permission_group_list_response import PermissionGroupListResponse

__all__ = ["PermissionGroupsResource", "AsyncPermissionGroupsResource"]


class PermissionGroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PermissionGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PermissionGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PermissionGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PermissionGroupsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        name: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[PermissionGroupListResponse]:
        """
        Find all available permission groups for API Tokens.

        Args:
          name: Filter by the name of the permission group. The value must be URL-encoded.

          scope: Filter by the scope of the permission group. The value must be URL-encoded.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/tokens/permission_groups",
            page=SyncSinglePage[PermissionGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "scope": scope,
                    },
                    permission_group_list_params.PermissionGroupListParams,
                ),
            ),
            model=PermissionGroupListResponse,
        )


class AsyncPermissionGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPermissionGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPermissionGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPermissionGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPermissionGroupsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        name: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PermissionGroupListResponse, AsyncSinglePage[PermissionGroupListResponse]]:
        """
        Find all available permission groups for API Tokens.

        Args:
          name: Filter by the name of the permission group. The value must be URL-encoded.

          scope: Filter by the scope of the permission group. The value must be URL-encoded.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/tokens/permission_groups",
            page=AsyncSinglePage[PermissionGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "scope": scope,
                    },
                    permission_group_list_params.PermissionGroupListParams,
                ),
            ),
            model=PermissionGroupListResponse,
        )


class PermissionGroupsResourceWithRawResponse:
    def __init__(self, permission_groups: PermissionGroupsResource) -> None:
        self._permission_groups = permission_groups

        self.list = to_raw_response_wrapper(
            permission_groups.list,
        )


class AsyncPermissionGroupsResourceWithRawResponse:
    def __init__(self, permission_groups: AsyncPermissionGroupsResource) -> None:
        self._permission_groups = permission_groups

        self.list = async_to_raw_response_wrapper(
            permission_groups.list,
        )


class PermissionGroupsResourceWithStreamingResponse:
    def __init__(self, permission_groups: PermissionGroupsResource) -> None:
        self._permission_groups = permission_groups

        self.list = to_streamed_response_wrapper(
            permission_groups.list,
        )


class AsyncPermissionGroupsResourceWithStreamingResponse:
    def __init__(self, permission_groups: AsyncPermissionGroupsResource) -> None:
        self._permission_groups = permission_groups

        self.list = async_to_streamed_response_wrapper(
            permission_groups.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .pcaps import (
    PCAPsResource,
    AsyncPCAPsResource,
    PCAPsResourceWithRawResponse,
    AsyncPCAPsResourceWithRawResponse,
    PCAPsResourceWithStreamingResponse,
    AsyncPCAPsResourceWithStreamingResponse,
)
from .download import (
    DownloadResource,
    AsyncDownloadResource,
    DownloadResourceWithRawResponse,
    AsyncDownloadResourceWithRawResponse,
    DownloadResourceWithStreamingResponse,
    AsyncDownloadResourceWithStreamingResponse,
)
from .ownership import (
    OwnershipResource,
    AsyncOwnershipResource,
    OwnershipResourceWithRawResponse,
    AsyncOwnershipResourceWithRawResponse,
    OwnershipResourceWithStreamingResponse,
    AsyncOwnershipResourceWithStreamingResponse,
)

__all__ = [
    "OwnershipResource",
    "AsyncOwnershipResource",
    "OwnershipResourceWithRawResponse",
    "AsyncOwnershipResourceWithRawResponse",
    "OwnershipResourceWithStreamingResponse",
    "AsyncOwnershipResourceWithStreamingResponse",
    "DownloadResource",
    "AsyncDownloadResource",
    "DownloadResourceWithRawResponse",
    "AsyncDownloadResourceWithRawResponse",
    "DownloadResourceWithStreamingResponse",
    "AsyncDownloadResourceWithStreamingResponse",
    "PCAPsResource",
    "AsyncPCAPsResource",
    "PCAPsResourceWithRawResponse",
    "AsyncPCAPsResourceWithRawResponse",
    "PCAPsResourceWithStreamingResponse",
    "AsyncPCAPsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["Sinkhole"]


class Sinkhole(BaseModel):
    id: Optional[int] = None
    """The unique identifier for the sinkhole"""

    account_tag: Optional[str] = None
    """The account tag that owns this sinkhole"""

    created_on: Optional[datetime] = None
    """The date and time when the sinkhole was created"""

    modified_on: Optional[datetime] = None
    """The date and time when the sinkhole was last modified"""

    name: Optional[str] = None
    """The name of the sinkhole"""

    r2_bucket: Optional[str] = None
    """The name of the R2 bucket to store results"""

    r2_id: Optional[str] = None
    """The id of the R2 instance"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["DetectionCreateResponse"]


class DetectionCreateResponse(BaseModel):
    """
    Defines a custom set of username/password expressions to match Leaked Credential Checks on.
    """

    id: Optional[str] = None
    """Defines the unique ID for this custom detection."""

    password: Optional[str] = None
    """Defines ehe ruleset expression to use in matching the password in a request."""

    username: Optional[str] = None
    """Defines the ruleset expression to use in matching the username in a request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WebhookGetWebhookByIDResponse", "Data"]


class Data(BaseModel):
    id: str
    """ID of the webhook"""

    created_at: datetime
    """Timestamp when this webhook was created"""

    enabled: bool
    """Set to true if the webhook is active"""

    events: List[
        Literal[
            "meeting.started",
            "meeting.ended",
            "meeting.participantJoined",
            "meeting.participantLeft",
            "meeting.chatSynced",
            "recording.statusUpdate",
            "livestreaming.statusUpdate",
            "meeting.transcript",
            "meeting.summary",
        ]
    ]
    """Events this webhook will send updates for"""

    name: str
    """Name of the webhook"""

    updated_at: datetime
    """Timestamp when this webhook was updated"""

    url: str
    """URL the webhook will send events to"""


class WebhookGetWebhookByIDResponse(BaseModel):
    data: Data

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ToMarkdownSupportedResponse"]


class ToMarkdownSupportedResponse(BaseModel):
    extension: str

    mime_type: str = FieldInfo(alias="mimeType")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .entry_get_response import EntryGetResponse as EntryGetResponse
from .entry_create_params import EntryCreateParams as EntryCreateParams
from .entry_list_response import EntryListResponse as EntryListResponse
from .entry_update_params import EntryUpdateParams as EntryUpdateParams
from .entry_create_response import EntryCreateResponse as EntryCreateResponse
from .entry_delete_response import EntryDeleteResponse as EntryDeleteResponse
from .entry_update_response import EntryUpdateResponse as EntryUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .resources import (
    ResourcesResource,
    AsyncResourcesResource,
    ResourcesResourceWithRawResponse,
    AsyncResourcesResourceWithRawResponse,
    ResourcesResourceWithStreamingResponse,
    AsyncResourcesResourceWithStreamingResponse,
)
from .recipients import (
    RecipientsResource,
    AsyncRecipientsResource,
    RecipientsResourceWithRawResponse,
    AsyncRecipientsResourceWithRawResponse,
    RecipientsResourceWithStreamingResponse,
    AsyncRecipientsResourceWithStreamingResponse,
)
from .resource_sharing import (
    ResourceSharingResource,
    AsyncResourceSharingResource,
    ResourceSharingResourceWithRawResponse,
    AsyncResourceSharingResourceWithRawResponse,
    ResourceSharingResourceWithStreamingResponse,
    AsyncResourceSharingResourceWithStreamingResponse,
)

__all__ = [
    "RecipientsResource",
    "AsyncRecipientsResource",
    "RecipientsResourceWithRawResponse",
    "AsyncRecipientsResourceWithRawResponse",
    "RecipientsResourceWithStreamingResponse",
    "AsyncRecipientsResourceWithStreamingResponse",
    "ResourcesResource",
    "AsyncResourcesResource",
    "ResourcesResourceWithRawResponse",
    "AsyncResourcesResourceWithRawResponse",
    "ResourcesResourceWithStreamingResponse",
    "AsyncResourcesResourceWithStreamingResponse",
    "ResourceSharingResource",
    "AsyncResourceSharingResource",
    "ResourceSharingResourceWithRawResponse",
    "AsyncResourceSharingResourceWithRawResponse",
    "ResourceSharingResourceWithStreamingResponse",
    "AsyncResourceSharingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ....._models import BaseModel

__all__ = ["GlobalWARPOverrideCreateResponse"]


class GlobalWARPOverrideCreateResponse(BaseModel):
    disconnect: Optional[bool] = None
    """Disconnects all devices on the account using Global WARP override."""

    timestamp: Optional[datetime] = None
    """When the Global WARP override state was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .outputs import (
    OutputsResource,
    AsyncOutputsResource,
    OutputsResourceWithRawResponse,
    AsyncOutputsResourceWithRawResponse,
    OutputsResourceWithStreamingResponse,
    AsyncOutputsResourceWithStreamingResponse,
)
from .live_inputs import (
    LiveInputsResource,
    AsyncLiveInputsResource,
    LiveInputsResourceWithRawResponse,
    AsyncLiveInputsResourceWithRawResponse,
    LiveInputsResourceWithStreamingResponse,
    AsyncLiveInputsResourceWithStreamingResponse,
)

__all__ = [
    "OutputsResource",
    "AsyncOutputsResource",
    "OutputsResourceWithRawResponse",
    "AsyncOutputsResourceWithRawResponse",
    "OutputsResourceWithStreamingResponse",
    "AsyncOutputsResourceWithStreamingResponse",
    "LiveInputsResource",
    "AsyncLiveInputsResource",
    "LiveInputsResourceWithRawResponse",
    "AsyncLiveInputsResourceWithRawResponse",
    "LiveInputsResourceWithStreamingResponse",
    "AsyncLiveInputsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_transit.sites import (
    lan_edit_params,
    lan_create_params,
    lan_update_params,
)
from ....types.magic_transit.sites.lan import LAN
from ....types.magic_transit.sites.nat_param import NatParam
from ....types.magic_transit.sites.routed_subnet_param import RoutedSubnetParam
from ....types.magic_transit.sites.lan_static_addressing_param import LANStaticAddressingParam

__all__ = ["LANsResource", "AsyncLANsResource"]


class LANsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LANsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LANsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LANsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LANsResourceWithStreamingResponse(self)

    def create(
        self,
        site_id: str,
        *,
        account_id: str,
        bond_id: int | Omit = omit,
        ha_link: bool | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[LAN]:
        """Creates a new Site LAN.

        If the site is in high availability mode,
        static_addressing is required along with secondary and virtual address.

        Args:
          account_id: Identifier

          site_id: Identifier

          ha_link: mark true to use this LAN for HA probing. only works for site with HA turned on.
              only one LAN can be set as the ha_link.

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/lans", account_id=account_id, site_id=site_id),
            page=SyncSinglePage[LAN],
            body=maybe_transform(
                {
                    "bond_id": bond_id,
                    "ha_link": ha_link,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_create_params.LANCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LAN,
            method="post",
        )

    def update(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        bond_id: int | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Update a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            body=maybe_transform(
                {
                    "bond_id": bond_id,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_update_params.LANUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    def list(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[LAN]:
        """
        Lists Site LANs associated with an account.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/lans", account_id=account_id, site_id=site_id),
            page=SyncSinglePage[LAN],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LAN,
        )

    def delete(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Remove a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    def edit(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        bond_id: int | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Patch a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            body=maybe_transform(
                {
                    "bond_id": bond_id,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_edit_params.LANEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    def get(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Get a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )


class AsyncLANsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLANsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLANsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLANsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLANsResourceWithStreamingResponse(self)

    def create(
        self,
        site_id: str,
        *,
        account_id: str,
        bond_id: int | Omit = omit,
        ha_link: bool | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LAN, AsyncSinglePage[LAN]]:
        """Creates a new Site LAN.

        If the site is in high availability mode,
        static_addressing is required along with secondary and virtual address.

        Args:
          account_id: Identifier

          site_id: Identifier

          ha_link: mark true to use this LAN for HA probing. only works for site with HA turned on.
              only one LAN can be set as the ha_link.

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/lans", account_id=account_id, site_id=site_id),
            page=AsyncSinglePage[LAN],
            body=maybe_transform(
                {
                    "bond_id": bond_id,
                    "ha_link": ha_link,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_create_params.LANCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LAN,
            method="post",
        )

    async def update(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        bond_id: int | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Update a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            body=await async_maybe_transform(
                {
                    "bond_id": bond_id,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_update_params.LANUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    def list(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LAN, AsyncSinglePage[LAN]]:
        """
        Lists Site LANs associated with an account.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites/{site_id}/lans", account_id=account_id, site_id=site_id),
            page=AsyncSinglePage[LAN],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=LAN,
        )

    async def delete(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Remove a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    async def edit(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        bond_id: int | Omit = omit,
        is_breakout: bool | Omit = omit,
        is_prioritized: bool | Omit = omit,
        name: str | Omit = omit,
        nat: NatParam | Omit = omit,
        physport: int | Omit = omit,
        routed_subnets: Iterable[RoutedSubnetParam] | Omit = omit,
        static_addressing: LANStaticAddressingParam | Omit = omit,
        vlan_tag: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Patch a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          is_breakout: mark true to use this LAN for source-based breakout traffic

          is_prioritized: mark true to use this LAN for source-based prioritized traffic

          static_addressing: If the site is not configured in high availability mode, this configuration is
              optional (if omitted, use DHCP). However, if in high availability mode,
              static_address is required along with secondary and virtual address.

          vlan_tag: VLAN ID. Use zero for untagged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            body=await async_maybe_transform(
                {
                    "bond_id": bond_id,
                    "is_breakout": is_breakout,
                    "is_prioritized": is_prioritized,
                    "name": name,
                    "nat": nat,
                    "physport": physport,
                    "routed_subnets": routed_subnets,
                    "static_addressing": static_addressing,
                    "vlan_tag": vlan_tag,
                },
                lan_edit_params.LANEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )

    async def get(
        self,
        lan_id: str,
        *,
        account_id: str,
        site_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LAN:
        """
        Get a specific Site LAN.

        Args:
          account_id: Identifier

          site_id: Identifier

          lan_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        if not lan_id:
            raise ValueError(f"Expected a non-empty value for `lan_id` but received {lan_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/sites/{site_id}/lans/{lan_id}",
                account_id=account_id,
                site_id=site_id,
                lan_id=lan_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[LAN]._unwrapper,
            ),
            cast_to=cast(Type[LAN], ResultWrapper[LAN]),
        )


class LANsResourceWithRawResponse:
    def __init__(self, lans: LANsResource) -> None:
        self._lans = lans

        self.create = to_raw_response_wrapper(
            lans.create,
        )
        self.update = to_raw_response_wrapper(
            lans.update,
        )
        self.list = to_raw_response_wrapper(
            lans.list,
        )
        self.delete = to_raw_response_wrapper(
            lans.delete,
        )
        self.edit = to_raw_response_wrapper(
            lans.edit,
        )
        self.get = to_raw_response_wrapper(
            lans.get,
        )


class AsyncLANsResourceWithRawResponse:
    def __init__(self, lans: AsyncLANsResource) -> None:
        self._lans = lans

        self.create = async_to_raw_response_wrapper(
            lans.create,
        )
        self.update = async_to_raw_response_wrapper(
            lans.update,
        )
        self.list = async_to_raw_response_wrapper(
            lans.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lans.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            lans.edit,
        )
        self.get = async_to_raw_response_wrapper(
            lans.get,
        )


class LANsResourceWithStreamingResponse:
    def __init__(self, lans: LANsResource) -> None:
        self._lans = lans

        self.create = to_streamed_response_wrapper(
            lans.create,
        )
        self.update = to_streamed_response_wrapper(
            lans.update,
        )
        self.list = to_streamed_response_wrapper(
            lans.list,
        )
        self.delete = to_streamed_response_wrapper(
            lans.delete,
        )
        self.edit = to_streamed_response_wrapper(
            lans.edit,
        )
        self.get = to_streamed_response_wrapper(
            lans.get,
        )


class AsyncLANsResourceWithStreamingResponse:
    def __init__(self, lans: AsyncLANsResource) -> None:
        self._lans = lans

        self.create = async_to_streamed_response_wrapper(
            lans.create,
        )
        self.update = async_to_streamed_response_wrapper(
            lans.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lans.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lans.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            lans.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            lans.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["Output"]


class Output(BaseModel):
    enabled: Optional[bool] = None
    """
    When enabled, live video streamed to the associated live input will be sent to
    the output URL. When disabled, live video will not be sent to the output URL,
    even when streaming to the associated live input. Use this to control precisely
    when you start and stop simulcasting to specific destinations like YouTube and
    Twitch.
    """

    stream_key: Optional[str] = FieldInfo(alias="streamKey", default=None)
    """The streamKey used to authenticate against an output's target."""

    uid: Optional[str] = None
    """A unique identifier for the output."""

    url: Optional[str] = None
    """The URL an output uses to restream."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.bots import web_crawler_summary_params, web_crawler_timeseries_groups_params
from ....types.radar.bots.web_crawler_summary_response import WebCrawlerSummaryResponse
from ....types.radar.bots.web_crawler_timeseries_groups_response import WebCrawlerTimeseriesGroupsResponse

__all__ = ["WebCrawlersResource", "AsyncWebCrawlersResource"]


class WebCrawlersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebCrawlersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WebCrawlersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebCrawlersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WebCrawlersResourceWithStreamingResponse(self)

    def summary(
        self,
        dimension: Literal[
            "CLIENT_TYPE",
            "USER_AGENT",
            "REFERER",
            "CRAWL_REFER_RATIO",
            "VERTICAL",
            "INDUSTRY",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        client_type: List[Literal["HUMAN", "NON_AI_BOT", "AI_BOT", "MIXED_PURPOSE"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerSummaryResponse:
        """
        Retrieves an aggregated summary of HTTP requests from crawlers, grouped by the
        specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          bot_operator: Filters results by bot operator.

          client_type: Filters results by agent type.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/bots/crawlers/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "bot_operator": bot_operator,
                        "client_type": client_type,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "vertical": vertical,
                    },
                    web_crawler_summary_params.WebCrawlerSummaryParams,
                ),
                post_parser=ResultWrapper[WebCrawlerSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[WebCrawlerSummaryResponse], ResultWrapper[WebCrawlerSummaryResponse]),
        )

    def timeseries_groups(
        self,
        dimension: Literal[
            "CLIENT_TYPE",
            "USER_AGENT",
            "REFERER",
            "CRAWL_REFER_RATIO",
            "VERTICAL",
            "INDUSTRY",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        client_type: List[Literal["HUMAN", "NON_AI_BOT", "AI_BOT", "MIXED_PURPOSE"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from crawlers, grouped by the
        specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          bot_operator: Filters results by bot operator.

          client_type: Filters results by agent type.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/bots/crawlers/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "bot_operator": bot_operator,
                        "client_type": client_type,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "vertical": vertical,
                    },
                    web_crawler_timeseries_groups_params.WebCrawlerTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[WebCrawlerTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[WebCrawlerTimeseriesGroupsResponse], ResultWrapper[WebCrawlerTimeseriesGroupsResponse]),
        )


class AsyncWebCrawlersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebCrawlersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebCrawlersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebCrawlersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWebCrawlersResourceWithStreamingResponse(self)

    async def summary(
        self,
        dimension: Literal[
            "CLIENT_TYPE",
            "USER_AGENT",
            "REFERER",
            "CRAWL_REFER_RATIO",
            "VERTICAL",
            "INDUSTRY",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        client_type: List[Literal["HUMAN", "NON_AI_BOT", "AI_BOT", "MIXED_PURPOSE"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerSummaryResponse:
        """
        Retrieves an aggregated summary of HTTP requests from crawlers, grouped by the
        specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          bot_operator: Filters results by bot operator.

          client_type: Filters results by agent type.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/bots/crawlers/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "bot_operator": bot_operator,
                        "client_type": client_type,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "vertical": vertical,
                    },
                    web_crawler_summary_params.WebCrawlerSummaryParams,
                ),
                post_parser=ResultWrapper[WebCrawlerSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[WebCrawlerSummaryResponse], ResultWrapper[WebCrawlerSummaryResponse]),
        )

    async def timeseries_groups(
        self,
        dimension: Literal[
            "CLIENT_TYPE",
            "USER_AGENT",
            "REFERER",
            "CRAWL_REFER_RATIO",
            "VERTICAL",
            "INDUSTRY",
            "RESPONSE_STATUS",
            "RESPONSE_STATUS_CATEGORY",
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        client_type: List[Literal["HUMAN", "NON_AI_BOT", "AI_BOT", "MIXED_PURPOSE"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        industry: SequenceNotStr[str] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        response_status: SequenceNotStr[str] | Omit = omit,
        response_status_category: List[
            Literal["INFORMATIONAL", "SUCCESS", "REDIRECTION", "CLIENT_ERROR", "SERVER_ERROR"]
        ]
        | Omit = omit,
        vertical: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from crawlers, grouped by the
        specified dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          bot_operator: Filters results by bot operator.

          client_type: Filters results by agent type.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          industry: Filters results by industry.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          response_status: Filters results by HTTP response status code (e.g. 200, 403, 404). Only
              [IANA-registered codes](https://www.iana.org/assignments/http-status-codes/http-status-codes.xhtml)
              are accepted.

          response_status_category: Filters results by HTTP response status code category.

          vertical: Filters results by vertical.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/bots/crawlers/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "bot_operator": bot_operator,
                        "client_type": client_type,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "industry": industry,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "response_status": response_status,
                        "response_status_category": response_status_category,
                        "vertical": vertical,
                    },
                    web_crawler_timeseries_groups_params.WebCrawlerTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[WebCrawlerTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[WebCrawlerTimeseriesGroupsResponse], ResultWrapper[WebCrawlerTimeseriesGroupsResponse]),
        )


class WebCrawlersResourceWithRawResponse:
    def __init__(self, web_crawlers: WebCrawlersResource) -> None:
        self._web_crawlers = web_crawlers

        self.summary = to_raw_response_wrapper(
            web_crawlers.summary,
        )
        self.timeseries_groups = to_raw_response_wrapper(
            web_crawlers.timeseries_groups,
        )


class AsyncWebCrawlersResourceWithRawResponse:
    def __init__(self, web_crawlers: AsyncWebCrawlersResource) -> None:
        self._web_crawlers = web_crawlers

        self.summary = async_to_raw_response_wrapper(
            web_crawlers.summary,
        )
        self.timeseries_groups = async_to_raw_response_wrapper(
            web_crawlers.timeseries_groups,
        )


class WebCrawlersResourceWithStreamingResponse:
    def __init__(self, web_crawlers: WebCrawlersResource) -> None:
        self._web_crawlers = web_crawlers

        self.summary = to_streamed_response_wrapper(
            web_crawlers.summary,
        )
        self.timeseries_groups = to_streamed_response_wrapper(
            web_crawlers.timeseries_groups,
        )


class AsyncWebCrawlersResourceWithStreamingResponse:
    def __init__(self, web_crawlers: AsyncWebCrawlersResource) -> None:
        self._web_crawlers = web_crawlers

        self.summary = async_to_streamed_response_wrapper(
            web_crawlers.summary,
        )
        self.timeseries_groups = async_to_streamed_response_wrapper(
            web_crawlers.timeseries_groups,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .references import (
    ReferencesResource,
    AsyncReferencesResource,
    ReferencesResourceWithRawResponse,
    AsyncReferencesResourceWithRawResponse,
    ReferencesResourceWithStreamingResponse,
    AsyncReferencesResourceWithStreamingResponse,
)
from .integrations import (
    IntegrationsResource,
    AsyncIntegrationsResource,
    IntegrationsResourceWithRawResponse,
    AsyncIntegrationsResourceWithRawResponse,
    IntegrationsResourceWithStreamingResponse,
    AsyncIntegrationsResourceWithStreamingResponse,
)

__all__ = [
    "ReferencesResource",
    "AsyncReferencesResource",
    "ReferencesResourceWithRawResponse",
    "AsyncReferencesResourceWithRawResponse",
    "ReferencesResourceWithStreamingResponse",
    "AsyncReferencesResourceWithStreamingResponse",
    "IntegrationsResource",
    "AsyncIntegrationsResource",
    "IntegrationsResourceWithRawResponse",
    "AsyncIntegrationsResourceWithRawResponse",
    "IntegrationsResourceWithStreamingResponse",
    "AsyncIntegrationsResourceWithStreamingResponse",
]
